@page{size:auto;margin:1.4cm}@media print{thead{display:table-header-group}body,html{height:auto;min-height:0}#pagemargin{margin:5px;min-height:0}}blockquote,body,dd,div,dl,dt,fieldset,form,h1,h2,h3,h4,h5,h6,html,input,p,pre,td,th{margin:0;padding:0}body,html,img{border:0}body,html{height:auto;width:100%}address,caption,cite,code,dfn,th,var{font-style:normal;font-weight:400}caption,th{text-align:left}h1,h2,h3,h4,h5,h6{font-size:100%}q:after,q:before{content:""}.nowrap{white-space:nowrap}.wrap{white-space:normal}.fullwidth,table{width:100%}table{border-collapse:collapse;font-size:11px;text-align:left}table th{font-weight:700;width:25%}table td,table th{padding:2px 3px 2px 0}