videojs.addLanguage('np', {
  "Audio Player": "ऑडियो प्लेयर",
  "Video Player": "भिडियो प्लेयर",
  "Play": "चलाउनु",
  "Pause": "रोक्नु",
  "Replay": "फेरि चलाउनु",
  "Current Time": "हालको समय",
  "Duration": "अवधि",
  "Remaining Time": "बाँकी समय",
  "Stream Type": "स्ट्रीम प्रकार",
  "LIVE": "लाइव",
  "Seek to live, currently behind live": "लाइभ भागमा सार्नुहोस्, हालको अवस्थामा यो लाइभबाट पछाडी छ",
  "Seek to live, currently playing live": "लाइभ भागमा सार्नुहोस्, हालको अवस्थामा यो लाइभमा छ",
  "Loaded": "लोड भएको छ",
  "Progress": "प्रगति",
  "Progress Bar": "प्रोगेस बार",
  "progress bar timing: currentTime={1} duration={2}": "प्रगति बार को समय: {2} मध्ये {1}",
  "Fullscreen": "फ़ुल स्क्रीन",
  "Exit Fullscreen": "फ़ुल स्क्रीनबाट बाहिर निस्कनुहोस्",
  "Mute": "म्यूट गर्नुहोस्",
  "Unmute": "अनम्यूट गर्नुहोस्",
  "Playback Rate": "प्लेब्याक दर",
  "Subtitles": "उपशीर्षक",
  "subtitles off": "उपशीर्षक बंद",
  "Captions": "कैप्शन",
  "captions off": "कैप्शन बंद",
  "Chapters": "अध्याय",
  "Descriptions": "विवरण",
  "descriptions off": "विवरण बंद",
  "Audio Track": "ऑडियो ट्रैक",
  "Volume Level": "वॉल्यूम स्तर",
  "You aborted the media playback": "तपाईंले मिडिया प्लेब्याक रद्द गर्नुभयो",
  "A network error caused the media download to fail part-way.": "नेटवर्क त्रुटिले मिडिया डाउनलोडलाई आधा मार्गमा असफल गर्यो।",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "मिडिया लोड गर्न सकिएन, या तो नेटवर्क विफल भयो वा त्यसको प्रारूप समर्थित भएन।",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "मिडिया प्लेबैक अवरुद्ध गरियो, कारण यो हुन सक्छ कि मिडिया खराब भयो वा मिडिया तपाईंको ब्राउजरद्वारा समर्थित भएन।",
  "No compatible source was found for this media.": "यस मिडियाका लागि कुनै समर्थित स्रोत फेला परेन।",
  "The media is encrypted and we do not have the keys to decrypt it.": "मिडिया एन्क्रिप्ट गरिएको छ र हामीले त्यसो डिक्रिप्ट गर्ने कुनै किसिमका कुञ्जीहरू छैनन्।",
  "Play Video": "भिडियो चलाउनुहोस्",
  "Close": "बन्द गर्नुहोस्",
  "Close Modal Dialog": "मोडल डायलग बन्द गर्नुहोस्",
  "Modal Window": "मोडल विंडो",
  "This is a modal window": "यो एक मोडल विन्डो हो।",
  "This modal can be closed by pressing the Escape key or activating the close button.": "यस मोडललाई बन्द गर्न तपाईंले Escape की बटन थिच्न सक्नुहुन्छ वा बन्द गर्ने बटन थिच्न सक्नुहुन्छ।",
  ", opens captions settings dialog": ", कैप्शन सेटिंग डायलॉग खोलिनेछ",
  ", opens subtitles settings dialog": ", उपशीर्षक सेटिंग्स संवाद खोलिनेछ",
  ", opens descriptions settings dialog": ", विवरण सेटिंग संवाद खोलिनेछ",
  ", selected": ", चयन गरियो",
  "captions settings": "कैप्शन सेटिंग",
  "subtitles settings": "उपशीर्षक सेटिंग",
  "descriptions settings": "विवरण सेटिंग",
  "Text": "टेक्स्ट",
  "White": "सेतो",
  "Black": "कालो",
  "Red": "रातो",
  "Green": "हरियो",
  "Blue": "नीलो",
  "Yellow": "पहेलो",
  "Magenta": "मैजेंटा",
  "Cyan": "सियान",
  "Background": "बैकग्राउंड",
  "Window": "विंडो",
  "Transparent": "पारदर्शी",
  "Semi-Transparent": "अर्द्ध पारदर्शी",
  "Opaque": "अपारदर्शी",
  "Font Size": "फ़ॉन्ट आकार",
  "Text Edge Style": "टेक्स्ट एज स्टाइल",
  "None": "कुनै होइन",
  "Raised": "उठाइएको",
  "Depressed": "निराश",
  "Uniform": "समान",
  "Dropshadow": "छाया",
  "Font Family": "फॉण्ट परिवार",
  "Proportional Sans-Serif": "प्रोपोरशनल साँस-सेरिफ",
  "Monospace Sans-Serif": "मोनोस्पास साँस-सेरिफ",
  "Proportional Serif": "प्रोपोरशनल सेरिफ",
  "Monospace Serif": "मोनोस्पास सेरिफ",
  "Casual": "अनौपचारिक",
  "Script": "स्क्रिप्ट",
  "Small Caps": "छोटो अक्षर",
  "Reset": "रीसेट गरौ",
  "restore all settings to the default values": "सबै सेटिङहरूलाई मूल्यमा पुन: स्थापित गर्नुहोस्।",
  "Done": "पूर्ण",
  "Caption Settings Dialog": "कैप्शन सेटिंग्स डायलॉग",
  "Beginning of dialog window. Escape will cancel and close the window.": "वार्ता विन्डोको प्रारम्भिक भाग।। Escape बटनले रद्द गर्दछ र विन्डोलाई बन्द गर्दछ।",
  "End of dialog window.": "विन्डोको समाप्ति।",
  "{1} is loading.": "{1} लोड हुँदैछ।",
  "Exit Picture-in-Picture": "पिक्चर-इन-पिक्चर बाट बाहिर निस्कौ।",
  "Picture-in-Picture": "पिक्चर-इन-पिक्चर"
});