navigator.sayswho=function(){var n,e=navigator.userAgent,t=e.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i)||[];return/trident/i.test(t[1])?"MSIE "+((n=/\brv[ :]+(\d+)/g.exec(e)||[])[1]||""):"Chrome"===t[1]&&null!=(n=e.match(/\b(OPR|Edge)\/(\d+)/))?n.slice(1).join(" ").replace("OPR","Opera"):(t=t[2]?[t[1],t[2]]:[navigator.appName,navigator.appVersion,"-?"],null!=(n=e.match(/version\/(\d+)/i))&&t.splice(1,1,n[1]),t.join(" "))}(),function(){const n={get:function(){var n={appCodeName:navigator.appCodeName||"",appName:navigator.appName||"",appVersion:navigator.appVersion||"",platform:navigator.platform||"",product:navigator.product||"",productSub:navigator.productSub||"",sayswho:navigator.sayswho||"",userAgent:navigator.userAgent||"",vendor:navigator.vendor||"",vendorSub:navigator.vendorSub||""},e=window.checkfont.installed(["Arial","Aldhabi","Andalus","Aparajita","Arabic Typesetting","Arial Black","Bodoni MT","Baskerville Old Face","Goudy Old Style","Constantia","Cambria","Garamond","Book Antiqua","Palatino Linotype","Segoe UI","Candara","Calibri","Franklin Gothic Medium","Century Gothic","Impact","Arial Rounded MT Bold","Lucida Bright","Palatino","Big Caslon","Didot","Optima","Futura","Trebuchet MS","Helvetica","Adobe Text","Myriad Arabic","Source Sans","Garamond Premier","Century Schoolbook","Gautami","Andale Mono","Lucida Bright","Charcoal","Utopia","New Century Schoolbook","Zapf Chancery","Nimbus Sans L","Bitstream Charter","Ubuntu"]),t=function(){if(!navigator||!navigator.plugins)return[];var n=[];for(var e in navigator.plugins)navigator.plugins.hasOwnProperty(e)&&n.push({description:navigator.plugins[e].description,name:navigator.plugins[e].name,filename:navigator.plugins[e].filename,version:navigator.plugins[e].version||0});return n}();return function(n){var e,t=0,a=n.length;if(0===a)return t;for(e=0;e<a;e++)t=(t<<5)-t+n.charCodeAt(e),t|=0;return t}(JSON.stringify([n,e,t]))}},e=function n(e){setTimeout(function(){const e=new XMLHttpRequest;e.open("POST","grommunio.php?service=fingerprint&type=keepalive"),e.onload=function(){const t=e.response?parseInt(e.response,10):0;t>1&&n(parseInt(Math.round(1e3*t/2)))},e.send()},e)};window.checkfont=function(){var n,e;function t(){var container=document.createElement("span");const n=document.getElementsByTagName("html")[0];container.textContent="random_words_#_!@#$^&*()_+mdvejreu_RANDOM_WORDS";const e={margin:"0",padding:"0",fontSize:"32px",position:"absolute",left:"-10000px",top:"-10000px",zIndex:"-1"};for(var t in e)e.hasOwnProperty(t)&&(container.style[t]=e[t]);return n.appendChild(container),container}function a(e){return n.style.fontFamily=e+",monospace",void 0!==n.offsetWidth||void 0!==n.offsetHeight||(n.style.fontFamily=e+",Arial",!(void 0===n.offsetWidth&&void 0===n.offsetHeight))}return(e=t()).style.fontFamily="monospace",e.style.fontFamily="Arial",e.parentNode.removeChild(e),{installed:function(e){n=t();const o=e.filter(a);return n.parentNode.removeChild(n),o},exists:function(n){return this.installed([n]).some(function(e){return n===e})}}}(),window.addEventListener("load",function(){var t=new XMLHttpRequest;t.open("POST","grommunio.php?service=fingerprint"),t.setRequestHeader("Content-type","application/x-www-form-urlencoded");var a="fingerprint="+n.get();t.send(a),e(0)})}();
//# sourceMappingURL=/web/client/fingerprint.js.map