videojs.addLanguage('et', {
  "Audio Player": "Heliesitaja",
  "Video Player": "Videoesitaja",
  "Play": "Esita",
  "Pause": "Paus",
  "Replay": "Esita uuesti",
  "Current Time": "Praegune aeg",
  "Duration": "Kestus",
  "Remaining Time": "Järelejäänud aeg",
  "Stream Type": "Voo tüüp",
  "LIVE": "OTSE",
  "Seek to live, currently behind live": "Mine üle otseedastusle, praegu reaalajast taga",
  "Seek to live, currently playing live": "Mine üle otseedastusle, praegu reaalajas ",
  "Loaded": "Laaditud",
  "Progress": "Edenemine",
  "Progress Bar": "Edenemisriba ",
  "progress bar timing: currentTime={1} duration={2}": "{1} / {2}",
  "Fullscreen": "Täisekraan",
  "Exit Fullscreen": "Välju täisekraanist",
  "Mute": "Vaigista",
  "Unmute": "Lõpeta vaigistus",
  "Playback Rate": "Taasesituse kiirus",
  "Subtitles": "Subtiitrid",
  "subtitles off": "subtiitrid väljas",
  "Captions": "Pealdised",
  "captions off": "pealdised väljas",
  "Chapters": "Peatükid",
  "Descriptions": "Kirjeldused",
  "descriptions off": "kirjeldused väljas",
  "Audio Track": "Helirada",
  "Volume Level": "Helitugevuse tase",
  "You aborted the media playback": "Katkestasid taasesituse",
  "A network error caused the media download to fail part-way.": "Võrguvea tõttu nurjus meediumifaili allalaadimine poole pealt.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "Seda meediumifaili ei õnnestunud laadida, kuna serveris või võrgus esines tõrge või kuna vormingut ei toetata.",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "Meediumifaili taasesitamine katkestati, kuna fail on rikutud või see kasutab funktsiooni, mida sinu brauser ei toeta.",
  "No compatible source was found for this media.": "Ei leitud selle meediumifailiga ühilduvat allikat.",
  "The media is encrypted and we do not have the keys to decrypt it.": "See meediumifail on krüpteeritud ja meil pole dekrüpteerimiseks vajalikku võtit.",
  "Play Video": "Esita video",
  "Close": "Sule",
  "Close Modal Dialog": "Sule modaaldialoog",
  "Modal Window": "Modaalaken",
  "This is a modal window": "See on modaalaken",
  "This modal can be closed by pressing the Escape key or activating the close button.": "Saad selle modaalelemendi sulgeda, vajutades paoklahvi või tehes sulgemisnupu aktiivseks.",
  ", opens captions settings dialog": ", avab pealdiste sätete dialoogi",
  ", opens subtitles settings dialog": ", avab subtiitrite sätete dialoogi",
  ", opens descriptions settings dialog": ", avab kirjelduste sätete dialoogi",
  ", selected": ", valitud",
  "captions settings": "pealdiste sätted",
  "subtitles settings": "subtiitrite sätted",
  "descriptions settings": "kirjelduste sätted",
  "Text": "Tekst",
  "White": "Valge",
  "Black": "Must",
  "Red": "Punane",
  "Green": "Roheline",
  "Blue": "Sinine",
  "Yellow": "Kollane",
  "Magenta": "Magneta",
  "Cyan": "Tsüaan",
  "Background": "Taust",
  "Window": "Aken",
  "Transparent": "Läbipaistev",
  "Semi-Transparent": "Poolläbipaistev",
  "Opaque": "Läbipaistmatu",
  "Font Size": "Fondi suurus",
  "Text Edge Style": "Tekstiserva stiil",
  "None": "Puudub",
  "Raised": "Kõrgem",
  "Depressed": "Madalam",
  "Uniform": "Ühtlane",
  "Drop shadow": "Langeva varjuga",
  "Font Family": "Fondipere",
  "Proportional Sans-Serif": "Seriifideta proportsionaalkiri",
  "Monospace Sans-Serif": "Seriifideta püsisammkiri",
  "Proportional Serif": "Seriifidega proportsionaalkiri",
  "Monospace Serif": "Seriifidega püsisammkiri",
  "Casual": "Sidumata kiri",
  "Script": "Käsikirjaline kiri",
  "Small Caps": "Kapiteelkiri",
  "Reset": "Lähtesta",
  "restore all settings to the default values": "taasta kõigi sätete vaikeväärtused",
  "Done": "Valmis",
  "Caption Settings Dialog": "Pealdiste sätete dialoog",
  "Beginning of dialog window. Escape will cancel and close the window.": "Dialoogiakna algus. Paoklahv loobub aknast ja suleb selle.",
  "End of dialog window.": "Dialoogiakna lõpp.",
  "{1} is loading.": "{1} laadimisel.",
  "Exit Picture-in-Picture": "Välju funktsioonist pilt pildis",
  "Picture-in-Picture": "Pilt pildis"
});