const resizeLoginBox=function(){const e=document.getElementsByTagName("body")[0],t=document.getElementById("bg"),n=document.getElementById("form-container"),i=document.getElementById("loading-mask")||e;function onResize(){if(!e)return;const l=.4375*i.clientHeight-n.clientHeight/2,o=(i.clientWidth-n.clientWidth)/2;n.style.top=l+"px",n.style.left=o+"px",t.style.width=i.clientWidth+"px",t.style.height=i.clientHeight+"px",t.style.top=-l+"px",t.style.left=-o+"px"}e.className+=(e.className.length>0?" ":"")+navigator.sayswho.split(" ")[0]+" "+navigator.sayswho.replace(" ","");var l=document.createElement("img");return l.onload=function(){n.style.visibility="visible"},l.src=window.getComputedStyle(i,!1).backgroundImage.slice(4,-1).replace(/"/g,""),onResize(),window.addEventListener("resize",onResize),onResize}();
//# sourceMappingURL=resize.js.map