.icon_mdm_settings{background-image:url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgZmlsbD0ibm9uZSI+PHBhdGggZmlsbD0iIzIxMjEyMSIgZD0iTTguMjU1IDdjLjk2NiAwIDEuNzUuNzgzIDEuNzUgMS43NXY5LjVBMS43NSAxLjc1IDAgMCAxIDguMjU1IDIwSDMuNzVBMS43NSAxLjc1IDAgMCAxIDIgMTguMjV2LTkuNUMyIDcuNzgzIDIuNzg0IDcgMy43NSA3em0wIDEuNUgzLjc1YS4yNS4yNSAwIDAgMC0uMjUuMjV2OS41YzAgLjEzOC4xMTIuMjUuMjUuMjVoNC41MDVhLjI1LjI1IDAgMCAwIC4yNS0uMjV2LTkuNWEuMjUuMjUgMCAwIDAtLjI1LS4yNU02LjI1MyAxNmEuNzUuNzUgMCAwIDEgLjEwMSAxLjQ5M2wtLjEwMS4wMDdoLS41YS43NS43NSAwIDAgMS0uMTAyLTEuNDkzTDUuNzUzIDE2ek0xOS43NSA0YTIuMjUgMi4yNSAwIDAgMSAyLjI0NSAyLjA5NkwyMiA2LjI1djguNWEyLjI1IDIuMjUgMCAwIDEtMi4wOTYgMi4yNDVMMTkuNzUgMTdIMTF2LTEuNWg4Ljc1YS43NS43NSAwIDAgMCAuNzQzLS42NDhsLjAwNy0uMTAydi04LjVhLjc1Ljc1IDAgMCAwLS42NDgtLjc0M0wxOS43NSA1LjVIOC4yNWEuNzUuNzUgMCAwIDAtLjcwNy41aC0xLjUzYTIuMjUgMi4yNSAwIDAgMSAyLjA3My0xLjk5NEw4LjI1IDR6bS00LjUgOWEuNzUuNzUgMCAwIDEgMCAxLjVoLTIuNWEuNzUuNzUgMCAxIDEgMC0xLjV6Ii8+PC9zdmc+)!important}.mdm-devicepanel.x-panel.tabpanel-container>.x-panel-bwrap>.x-panel-body{padding:0}.mdm-devicepanel.x-panel.tabpanel-container>.x-panel-bwrap>.x-panel-body>.x-tab-panel>.x-tab-panel-bwrap>.x-tab-panel-body{padding:0!important}.mdm-display-name{font-weight:700!important;padding:4px 0 2px}.mdm-device-panel{padding:5px 0 5px 10px}.mdm-field-sep{border-top:1px solid;border-color:#e6e6e6}.mdm-synchronize-panel{border:0;margin-left:10px;padding:9px 0 0}.mdm-managesharedfolder-button{padding-right:10px}.mdm-create-tree-panel-item{margin:0 0 5px}.mdm-synchronize-shared-panel{padding-top:9px}.mdm-autherror-bar .x-btn-small{background:#af4236!important;box-shadow:none;text-align:center;width:100%}.mdm-autherror-bar button{color:#fff!important;font-size:1.02em}.mdm-autherror-bar .x-btn-small:hover,.mdm-autherror-bar.x-btn-click>.x-btn-small,.mdm-autherror-bar.x-btn-over.x-btn-click.x-btn-focus>.x-btn-small,.mdm-autherror-bar.x-btn-over.x-btn-click>.x-btn-small,.mdm-autherror-bar.x-btn-over.x-btn-focus>.x-btn-small,.mdm-autherror-bar.x-btn-over>.x-btn-small{background:#b72b00!important;box-shadow:none}