*{box-sizing:border-box}body{margin:0}fieldset{border:1px solid silver;margin:0 2px;padding:.35em .625em .75em}table{border-collapse:collapse;border-spacing:0}td,th{padding:0}body{font-family:Roboto,sans-serif;font-size:14px;font-weight:300;line-height:22px}h1{color:#b10610;font-size:42px;margin-bottom:30px;margin-top:10px;padding-bottom:5px}h1,h2{line-height:44px}h2{color:#333;font-size:28px;font-weight:300}h3{font-size:21px;margin-bottom:10px;margin-top:20px}a{color:#31a1cd}h1 a{-webkit-text-decoration:none;text-decoration:none}h2 a{color:#333}a:visited{color:#6098a2}h2 a:visited{color:#333}a:hover{color:#b10610}hr{border:none;border-top:1px dashed #c9ea75;margin-bottom:30px;margin-top:30px}header{background:#eee}header a{color:#333;font-size:28px;font-weight:500;-webkit-text-decoration:none;text-decoration:none}.logo{padding:5px 10px}.logo img{border:0;vertical-align:middle}button,input,select{color:inherit;font:inherit}input[type=text],select{border:1px solid #bbb;border-radius:3px;line-height:22px;padding:5px 10px}nav{padding:5px}.btn,button,input[type=submit]{background:#4fa3ac;border:0;border-radius:2px;color:#fff;display:inline-block;padding:9px 15px;-webkit-text-decoration:none;text-decoration:none}a.btn:visited{color:#fff}.btn.disabled{background:#eee;color:#bbb}section{margin:40px 10px}section table{height:40px}.nodeTable tr{border-bottom:3px solid #fff}.nodeTable td{padding:10px}.nodeTable a{-webkit-text-decoration:none;text-decoration:none}.nodeTable .nameColumn{background:#ebf5f6;font-weight:700;min-width:200px;padding:10px 20px}.nodeTable .oi{color:#b10610}.propTable tr{height:40px}.propTable th{background:#f6f6f6;padding:0 10px;text-align:left}.propTable td{background:#eee;padding:0 10px}.propTable pre{background:#f8f8f8;font-size:80%}.actions{border:1px dotted #76baa6;margin-bottom:20px;padding:20px}.actions h3{border-bottom:1px solid #eee;margin-bottom:30px;margin-top:10px;padding-bottom:20px}.actions label{display:inline-block;line-height:40px;width:150px}.actions input[type=text],select{width:450px}.actions input[type=submit]{display:inline-block;margin-left:153px}footer{font-size:80%;padding:50px 0;text-align:center}ul.tree{list-style:none;margin:0;padding:0}ul.tree ul{border-left:4px solid #ccc;list-style:none;padding-left:10px}