#printlist{border-collapse:collapse;width:100%}#printlist th{background-color:#d3d3d3;font-size:13px;text-align:left}#printlist td,#printlist th{border:1px solid #ddd;color:#222;padding:8px}#printlist td{font-size:11px}#printlist tr:nth-child(2n){background-color:#f2f2f2}#printlistbottom{color:#222;font-size:12px;padding-top:5px;width:100%}#printlistbottom .right{float:right}