.vjs-theme-sea .vjs-big-play-button{background-color:hsla(0,0%,100%,.25);border:none;border-radius:30px;height:79px;left:50%;line-height:79px;margin:-51.5px auto 0 -39.5px;-o-object-fit:contain;object-fit:contain;top:50%;width:103px}