.icon_kendox_settings{background-position:6px;background-repeat:no-repeat}.icon_kendox_settings,.icon_kendox_upload{background-image:url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDIzNSA0MzAiPjxwYXRoIGZpbGw9IiMyMTIxMjEiIGZpbGwtcnVsZT0iZXZlbm9kZCIgZD0iTTE5LjAyMSAwdjIxNS42OThoODYuOTUxTDAgNDI5LjY1OWgxMDQuMjgybDEzMC40MjUtMjM4LjQzN1YweiIvPjwvc3ZnPg==)}.icon_kendox_upload{background-position:50%!important;background-repeat:no-repeat!important}.icon_kendox_main{background-image:url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCIgdmlld0JveD0iMCAwIDIzNSA0MzAiPjxwYXRoIGZpbGw9IiMyMTIxMjEiIGZpbGwtcnVsZT0iZXZlbm9kZCIgZD0iTTE5LjAyMSAwdjIxNS42OThoODYuOTUxTDAgNDI5LjY1OWgxMDQuMjgybDEzMC40MjUtMjM4LjQzN1YweiIvPjwvc3ZnPg==);background-position:50%}