.icon_16_Seafile{background-image:url(../icons/seafile16.png)!important;background-repeat:no-repeat}.icon_24_Seafile{background-image:url(../icons/seafile24.png)!important;background-repeat:no-repeat}.icon_256_Seafile{background-image:url(../icons/seafile256.png)!important;background-repeat:no-repeat}.icon_16_logo_Seafile{background-image:url(../icons/seafile_logo16.png)!important;background-repeat:no-repeat}.shareicon_16_user{background-image:url(../icons/usericon.png)!important}.shareicon_16_group,.shareicon_16_user{background-position:50%;background-repeat:no-repeat}.shareicon_16_group{background-image:url(../icons/groupicon.png)!important}.shareicon_16_yes{background-image:url(../icons/icon_yes.png)!important}.shareicon_16_no,.shareicon_16_yes{background-position:50%;background-repeat:no-repeat}.shareicon_16_no{background-image:url(../icons/icon_no.png)!important}.icon_copy_clipboard{background-image:url(../icons/clipboard.png)!important;background-position:50%;background-repeat:no-repeat}.ugpredic_search_item{border:1px solid;border-color:#fff #fff #eee;color:#555;font:normal 11px tahoma,arial,helvetica,sans-serif;padding:3px 10px;white-space:normal}.ugpredic_search_item h3{color:#222;display:block;font:inherit;font-weight:700}.ugpredic_search_item h3 span{clear:none;display:block;float:left;font-weight:400;padding:0 10px 0 0;width:20px}input[name=permissionChange],input[name=permissionCreate],input[name=permissionDelete],input[name=permissionShare]{margin-top:5px}