videojs.addLanguage('el', {
  "Audio Player": "Αναπαραγωγέας Ήχου",
  "Video Player": "Αναπαραγωγέας Βίντεο",
  "Play": "Aναπαραγωγή",
  "Pause": "Παύση",
  "Replay": "Επανάληψη",
  "Current Time": "Τρέχων χρόνος",
  "Duration": "Συνολικός χρόνος",
  "Remaining Time": "Υπολοιπόμενος χρόνος",
  "Stream Type": "Τύπος ροής",
  "LIVE": "ΖΩΝΤΑΝΑ",
  "Loaded": "Φόρτωση επιτυχής",
  "Progress": "Πρόοδος",
  "Progress Bar": "Μπάρα Προόδου",
  "Fullscreen": "Πλήρης οθόνη",
  "Exit Fullscreen": "Έξοδος από πλήρη οθόνη",
  "Mute": "Σίγαση",
  "Unmute": "Kατάργηση σίγασης",
  "Playback Rate": "Ρυθμός αναπαραγωγής",
  "Subtitles": "Υπότιτλοι",
  "subtitles off": "απόκρυψη υπότιτλων",
  "Captions": "Λεζάντες",
  "captions off": "απόκρυψη λεζάντων",
  "Chapters": "Κεφάλαια",
  "Close Modal Dialog": "Κλείσιμο Αναδυόμενου Παραθύρου",
  "Descriptions": "Περιγραφές",
  "descriptions off": "απόκρυψη περιγραφών",
  "Audio Track": "Ροή ήχου",
  "Volume Level": "Επίπεδο Ήχου",
  "You aborted the media playback": "Ακυρώσατε την αναπαραγωγή",
  "A network error caused the media download to fail part-way.": "Ένα σφάλμα δικτύου προκάλεσε την αποτυχία μεταφόρτωσης του αρχείου προς αναπαραγωγή.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "Το αρχείο προς αναπαραγωγή δεν ήταν δυνατό να φορτωθεί είτε γιατί υπήρξε σφάλμα στον διακομιστή ή το δίκτυο, είτε γιατί ο τύπος του αρχείου δεν υποστηρίζεται.",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "Η αναπαραγωγή ακυρώθηκε είτε λόγω κατεστραμμένου αρχείου, είτε γιατί το αρχείο απαιτεί λειτουργίες που δεν υποστηρίζονται από το πρόγραμμα περιήγησης που χρησιμοποιείτε.",
  "No compatible source was found for this media.": "Δεν βρέθηκε συμβατή πηγή αναπαραγωγής για το συγκεκριμένο αρχείο.",
  "The media is encrypted and we do not have the keys to decrypt it.": "Το αρχείο προς αναπαραγωγή είναι κρυπτογραφημένo και δεν υπάρχουν τα απαραίτητα κλειδιά αποκρυπτογράφησης.",
  "Play Video": "Αναπαραγωγή βίντεο",
  "Close": "Κλείσιμο",
  "Modal Window": "Aναδυόμενο παράθυρο",
  "This is a modal window": "Το παρών είναι ένα αναδυόμενο παράθυρο",
  "This modal can be closed by pressing the Escape key or activating the close button.": "Αυτό το παράθυρο μπορεί να εξαφανιστεί πατώντας το πλήκτρο Escape ή πατώντας το κουμπί κλεισίματος.",
  ", opens captions settings dialog": ", εμφανίζει τις ρυθμίσεις για τις λεζάντες",
  ", opens subtitles settings dialog": ", εμφανίζει τις ρυθμίσεις για τους υπότιτλους",
  ", opens descriptions settings dialog": ", εμφανίζει τις ρυθμίσεις για τις περιγραφές",
  ", selected": ", επιλεγμένο",
  "captions settings": "ρυμθίσεις λεζάντας",
  "subtitles settings": "ρυθμίσεις υποτίτλων",
  "descriptions settings": "ρυθμίσεις περιγραφών",
  "Text": "Κείμενο",
  "White": "Λευκό",
  "Black": "Μαύρο",
  "Red": "Κόκκινο",
  "Green": "Πράσινο",
  "Blue": "Μπλέ",
  "Yellow": "Κίτρινο",
  "Magenta": "Ματζέντα",
  "Cyan": "Κυανό",
  "Background": "Φόντο",
  "Window": "Παράθυρο",
  "Transparent": "Διάφανο",
  "Semi-Transparent": "Ημιδιάφανο",
  "Opaque": "Αδιαφανές",
  "Font Size": "Μέγεθος Γραμματοσειράς",
  "Text Edge Style": "Στυλ Περιθωρίου Κειμένου",
  "None": "Κανένα",
  "Uniform": "Ομοιόμορφο",
  "Drop shadow": "Σκίαση",
  "Font Family": "Οικογένεια Γραμματοσειράς",
  "Reset": "Επαναφορά",
  "restore all settings to the default values": "επαναφορά όλων των ρυθμίσεων στις προεπιλεγμένες τιμές",
  "Done": "Ολοκληρώθηκε",
  "Caption Settings Dialog": "Διάλογος Ρυθμίσεων Λεζάντας",
  "Beginning of dialog window. Escape will cancel and close the window.": "Αρχή του παραθύρου διαλόγου. Το Escape θα ακυρώσει και θα κλείσει το παράθυρο.",
  "End of dialog window.": "Τέλος του παραθύρου διαλόγου.",
  "{1} is loading.": "{1} φορτώνει.",
  "Exit Picture-in-Picture": "Έξοδος από την Εικόνα-μέσα-σε-Εικόνα",
  "Picture-in-Picture": "Εικόνα-μέσα-σε-Εικόνα",
  "No content": "Χωρίς περιεχόμενο",
  "Color": "Χρώμα",
  "Opacity": "Αδιαφάνεια",
  "Text Background": "Φόντο Κειμένου",
  "Caption Area Background": "Φόντο Περιοχής Λεζάντας",
  "Playing in Picture-in-Picture": "Αναπαραγωγή σε Εικόνα-μέσα-σε-Εικόνα",
  "Skip backward {1} seconds": "Μεταβείτε πίσω {1} δευτερόλεπτα",
  "Skip forward {1} seconds": "Μεταβείτε μπροστά {1} δευτερόλεπτα"
});