videojs.addLanguage('mr', {
  "Audio Player": "ध्वनि वादक",
  "Video Player": "चित्रवाणी वादक",
  "Play": "वाजवा",
  "Pause": "थांबा",
  "Replay": "पुन्हा वाजवा",
  "Current Time": "वर्तमान वेळ",
  "Duration": "कालावधी",
  "Remaining Time": "उर्वरित वेळ",
  "Stream Type": "प्रवाह प्रकार",
  "LIVE": "थेट प्रसारण",
  "Seek to live, currently behind live": "थेट प्रसारणकडे जा, चालू क्षणाला थेट प्रसारण पासुन मागे आहे",
  "Seek to live, currently playing live": "थेट प्रसारणकडे जा, चालू क्षणाला थेट प्रसारण आहे",
  "Loaded": "सज्ज",
  "Progress": "प्रगती",
  "Progress Bar": "प्रगती पट्टा",
  "progress bar timing: currentTime={1} duration={2}": "{2} पैकी {1}",
  "Fullscreen": "संपूर्ण पडदा",
  "Exit Fullscreen": "संपूर्ण पडद्यातून बाहेर पडा",
  "Mute": "मुक",
  "Unmute": "अमूक",
  "Playback Rate": "प्लेबॅक दर",
  "Subtitles": "उपशीर्षके",
  "subtitles off": "उपशीर्षके बंद",
  "Captions": "मथळे",
  "captions off": "मथळे बंद",
  "Chapters": "अध्याय",
  "Descriptions": "वर्णने",
  "descriptions off": "वर्णने बंद",
  "Audio Track": "ध्वनि मार्ग",
  "Volume Level": "आवाज पातळी",
  "You aborted the media playback": "तुम्ही मीडिया प्लेबॅक रद्द केला",
  "A network error caused the media download to fail part-way.": "नेटवर्क त्रुटीमुळे मीडिया डाउनलोड अर्ध्यात अयशस्वी झाला.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "मीडिया लोड करता आला नाही, एकतर सर्व्हर किंवा नेटवर्क अयशस्वी झाल्यामुळे किंवा फॉरमॅट समर्थित नसल्यामुळे.",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "मीडिया प्लेबॅक भ्रष्टाचाराच्या समस्येमुळे किंवा मीडियाने वापरलेल्या वैशिष्ट्यांमुळे तुमचा ब्राउझर सपोर्ट करत नसल्यामुळे रद्द करण्यात आला.",
  "No compatible source was found for this media.": "या मीडियासाठी कोणताही सुसंगत स्रोत सापडला नाही.",
  "The media is encrypted and we do not have the keys to decrypt it.": "मीडिया एन्क्रिप्ट केलेला आहे आणि तो डिक्रिप्ट करण्यासाठी आमच्याकडे कळा नाहीत.",
  "Play Video": "चित्रवाणी वाजवा",
  "Close": "बंद",
  "Close Modal Dialog": "संवादी प्रलेखन बंद करा",
  "Modal Window": "संवादी प्रलेखन",
  "This is a modal window": "हे एक संवादी प्रलेखन आहे",
  "This modal can be closed by pressing the Escape key or activating the close button.": "एस्केप कळ दाबून किंवा बंद कळ सक्रिय करून हे संवादी प्रलेखन बंद केले जाऊ शकते.",
  ", opens captions settings dialog": ", मथळे नियंत्रण संवाद प्रलेखन उघडतो",
  ", opens subtitles settings dialog": ", उपशीर्षक नियंत्रण संवाद प्रलेखन उघडतो",
  ", opens descriptions settings dialog": ", वर्णन नियंत्रण संवाद प्रलेखन उघडतो",
  ", selected": ", निवडले",
  "captions settings": "मथळे नियंत्रण",
  "subtitles settings": "उपशीर्षक नियंत्रण",
  "descriptions settings": "वर्णन नियंत्रण",
  "Text": "मजकूर",
  "White": "पांढरा",
  "Black": "काळा",
  "Red": "लाल",
  "Green": "हिरवा",
  "Blue": "निळा",
  "Yellow": "पिवळा",
  "Magenta": "किरमिजी तांबडा",
  "Cyan": "निळसर",
  "Background": "पृष्ठभाग",
  "Window": "चौकट",
  "Transparent": "पारदर्शक",
  "Semi-Transparent": "अर्ध-पारदर्शक",
  "Opaque": "अपारदर्शक",
  "Font Size": "अक्षराचा आकार",
  "Text Edge Style": "मजकूर एज शैली",
  "None": "काहीही नाही",
  "Raised": "वाढवले",
  "Depressed": "न दाबलेले",
  "Uniform": "एकसमान",
  "Drop shadow": "सावली सोडा",
  "Font Family": "अक्षर कुटुंब",
  "Proportional Sans-Serif": "आनुपातिक सॅन्स-सेरिफ",
  "Monospace Sans-Serif": "मोनोस्पेस सॅन्स-सेरिफ",
  "Proportional Serif": "आनुपातिक सेरिफ",
  "Monospace Serif": "मोनोस्पेस सेरिफ",
  "Casual": "प्रासंगिक",
  "Script": "लेख",
  "Small Caps": "लहान अक्षरे",
  "Reset": "पुन्हा कर",
  "restore all settings to the default values": "सर्व नियंत्रण डीफॉल्ट मूल्यांवर पुनर्संचयित करा",
  "Done": "झाले",
  "Caption Settings Dialog": "मथळा नियंत्रण संवाद प्रलेखन",
  "Beginning of dialog window. Escape will cancel and close the window.": "डायलॉग विंडोची सुरुवात. Escape रद्द करेल आणि विंडो बंद करेल.",
  "End of dialog window.": "डायलॉग विंडोचा शेवट.",
  "{1} is loading.": "{1} लोड होत आहे.",
  "Exit Picture-in-Picture": "पिक्चर-इन-पिक्चरमधून बाहेर पडा",
  "Picture-in-Picture": "पिक्चर-इन-पिक्चर",
  "No content": "सामग्री नाही",
  "Color": "रंग",
  "Opacity": "अपारदर्शकता",
  "Text Background": "मजकूर पार्श्वभूमी",
  "Caption Area Background": "मथळा क्षेत्र पार्श्वभूमी",
  "Playing in Picture-in-Picture": "पिक्चर-इन-पिक्चरमध्ये वाजवत आहे",
  "Skip backward {1} seconds": "मागे जा {1} सेकंद",
  "Skip forward {1} seconds": "पुढे जा {1} सेकंद"
});