<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => 'a181f852c811cb0eefcc629d09d22af88c79b6f7',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => 'a181f852c811cb0eefcc629d09d22af88c79b6f7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.8.0',
            'version' => '2.8.0.0',
            'reference' => '21dc724a0583619cd1652f673303492272778051',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/serializable-closure' => array(
            'pretty_version' => 'v2.0.5',
            'version' => '2.0.5.0',
            'reference' => '3832547db6e0e2f8bb03d4093857b378c66eceed',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/serializable-closure',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/constant_time_encoding' => array(
            'pretty_version' => 'v3.1.3',
            'version' => '3.1.3.0',
            'reference' => 'd5b01a39b3415c2cd581d3bd3a3575c1ebbd8e77',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/constant_time_encoding',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'pretty_version' => 'v9.99.100',
            'version' => '9.99.100.0',
            'reference' => '996434e5492cb4c3edcb9168db6fbb1359ef965a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/random_compat',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-di/invoker' => array(
            'pretty_version' => '2.3.7',
            'version' => '2.3.7.0',
            'reference' => '3c1ddfdef181431fbc4be83378f6d036d59e81e1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-di/invoker',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-di/php-di' => array(
            'pretty_version' => '7.1.1',
            'version' => '7.1.1.0',
            'reference' => 'f88054cc052e40dbe7b383c8817c19442d480352',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-di/php-di',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpseclib/phpseclib' => array(
            'pretty_version' => '3.0.46',
            'version' => '3.0.46.0',
            'reference' => '56483a7de62a6c2a6635e42e93b8a9e25d4f0ec6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/phpseclib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'pimple/pimple' => array(
            'pretty_version' => 'v3.5.0',
            'version' => '3.5.0.0',
            'reference' => 'a94b3a4db7fb774b3d78dad2315ddc07629e1bed',
            'type' => 'library',
            'install_path' => __DIR__ . '/../pimple/pimple',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => 'c71ecc56dfe541dbd90c5360474fbc405f8d5963',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '^1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '2.0',
            'version' => '2.0.0.0',
            'reference' => '402d35bcb92c70c026d1a6a9883f06b2ead23d71',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f16e1d5863e37f8d8c2a01719f5b34baa2b714d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-iconv' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '5f3b930437ae03ae5dff61269024d8ea1b3774aa',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-iconv',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '6d857f4d76bd4b343eac26d6b539585d2bc56493',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'zbateson/mail-mime-parser' => array(
            'pretty_version' => '3.0.4',
            'version' => '3.0.4.0',
            'reference' => 'f0ccec9290a5b9cf014d7b7ea3401d2a4a626e9a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../zbateson/mail-mime-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'zbateson/mb-wrapper' => array(
            'pretty_version' => '2.0.1',
            'version' => '2.0.1.0',
            'reference' => '50a14c0c9537f978a61cde9fdc192a0267cc9cff',
            'type' => 'library',
            'install_path' => __DIR__ . '/../zbateson/mb-wrapper',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'zbateson/stream-decorators' => array(
            'pretty_version' => '2.1.1',
            'version' => '2.1.1.0',
            'reference' => '32a2a62fb0f26313395c996ebd658d33c3f9c4e5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../zbateson/stream-decorators',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
