.leaflet-control-geosearch *,.leaflet-control-geosearch :after,.leaflet-control-geosearch :before{box-sizing:border-box}.leaflet-control-geosearch .leaflet-bar-part{border-bottom:none;border-radius:4px}.leaflet-control-geosearch a.leaflet-bar-part:after,.leaflet-control-geosearch a.leaflet-bar-part:before{content:"";display:block;position:absolute}.leaflet-control-geosearch a.leaflet-bar-part:before{border-top:2px solid #555;left:13px;top:15px;transform:rotate(45deg);width:6px}.leaflet-control-geosearch a.leaflet-bar-part:after{border:2px solid #555;border-radius:50%;height:8px;left:8px;top:8px;width:8px}.leaflet-control-geosearch.error a.leaflet-bar-part:before,.leaflet-control-geosearch.pending a.leaflet-bar-part:before{display:none}.leaflet-control-geosearch.error a.leaflet-bar-part:after,.leaflet-control-geosearch.pending a.leaflet-bar-part:after{border-radius:50%;height:18px;left:50%;margin:-9px 0 0 -9px;top:50%;width:18px}.leaflet-control-geosearch.pending a.leaflet-bar-part:after{animation:spin 1s linear infinite;border:2px solid #555;border-top-color:#f3f3f3;content:""}.leaflet-control-geosearch.error a.leaflet-bar-part:after{border:none;content:"!";font-size:18px;font-weight:600;line-height:normal}.leaflet-control-geosearch form{background-clip:padding-box;background-color:#fff;border-radius:0 4px 4px 0;box-shadow:0 1px 5px rgba(0,0,0,.65);display:none;height:auto;left:36px;margin:0;padding:0 8px;position:absolute;top:0;z-index:-1}.leaflet-geosearch-button form.open{border-radius:0 4px 4px 4px}.leaflet-control-geosearch.active form{display:block}.leaflet-geosearch-button.active .leaflet-bar-part{border-radius:4px 0 0 4px;width:36px}.leaflet-geosearch-button form{max-width:350px}.leaflet-control-geosearch form input{border:none;border-radius:0 4px 4px 0;font-size:12px;height:26px;margin:0;min-width:200px;outline:none;padding:0;text-indent:8px;width:100%}.leaflet-touch .leaflet-geosearch-bar form{border:2px solid rgba(0,0,0,.2);box-shadow:none}.leaflet-touch .leaflet-geosearch-bar form input{height:30px}.leaflet-control-geosearch .results{background:#fff}.leaflet-control-geosearch .results>*{border:1px solid transparent;line-height:24px;overflow:hidden;padding:0 8px;text-overflow:ellipsis;white-space:nowrap}.leaflet-control-geosearch .results.active{border-top:1px solid #c6c6c6;padding:8px 0}.leaflet-control-geosearch .results>.active,.leaflet-control-geosearch .results>:hover{background-color:#f8f8f8;border-color:#c6c6c6;cursor:pointer}.leaflet-control-geosearch .results.active:after{bottom:-2px;content:"";display:block;left:-2px;position:absolute;top:30px;width:0}.leaflet-touch .leaflet-control-geosearch .results.active:after{border-left:2px solid rgba(0,0,0,.2)}@keyframes spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}.leaflet-bottom .leaflet-geosearch-bar,.leaflet-top .leaflet-geosearch-bar{display:none}.leaflet-geosearch-bar{cursor:auto;display:block;height:auto;margin:10px auto 0;max-width:calc(100% - 120px);position:relative;width:400px;z-index:1000}.leaflet-geosearch-bar form{border-radius:4px;display:block;left:0;position:relative;top:0}.leaflet-geosearch-bar form input{min-width:100%;width:100%}.leaflet-geosearch-bar .results.active:after{opacity:.2}.leaflet-right .leaflet-control-geosearch form{border-left:inherit;border-radius:4px 0 0 4px;border-right:none;left:auto;right:28px}.leaflet-control-geosearch a.reset{background-color:#fff;border:none;border-radius:0 4px 4px 0;color:#000;cursor:pointer;font-weight:700;line-height:26px;padding:0 8px;position:absolute;right:0;-webkit-text-decoration:none;text-decoration:none;top:0}.leaflet-touch .leaflet-control-geosearch a.reset{line-height:30px}.leaflet-control-geosearch a.reset:hover{background:#f5f5f5}