videojs.addLanguage('bn', {
  "Audio Player": "অডিও প্লেয়ার",
  "Video Player": "ভিডিও প্লেয়ার",
  "Play": "প্লে করুন",
  "Pause": "বিরাম",
  "Replay": "রিপ্লে করুন",
  "Current Time": "বর্তমান সময়",
  "Duration": "ব্যাপ্তিকাল",
  "Remaining Time": "অবশিষ্ট সময়",
  "Stream Type": "স্ট্রিমের ধরন",
  "LIVE": "লাইভ",
  "Seek to live, currently behind live": "লাইভ দেখুন, বর্তমানে লাইভের পিছনে আছে",
  "Seek to live, currently playing live": "লাইভ দেখুন, বর্তমানে লাইভে আছে",
  "Loaded": "লোড হয়েছে",
  "Progress": "প্রোগ্রেস",
  "Progress Bar": "প্রোগ্রেস বার",
  "progress bar timing: currentTime={1} duration={2}": "{2} এর {1}",
  "Fullscreen": "পূর্ণ স্ক্রীন",
  "Non-Fullscreen": "পূর্ণ স্ক্রীন থেকে প্রস্থান করুন",
  "Mute": "মিউট",
  "Unmute": "আনমিউট",
  "Playback Rate": "প্লেব্যাক রেট",
  "Subtitles": "সাবটাইটেল",
  "subtitles off": "সাবটাইটেল বন্ধ করুন",
  "Captions": "ক্যাপশন",
  "captions off": "ক্যাপশন বন্ধ করুন",
  "Chapters": "অধ্যায়",
  "Descriptions": "বর্ণনা",
  "descriptions off": "বর্ণনা বন্ধ করুন",
  "Audio Track": "অডিও গান",
  "Volume Level": "ভলিউম লেভেল",
  "You aborted the media playback": "আপনি মিডিয়া প্লেব্যাক বাতিল করেছেন",
  "A network error caused the media download to fail part-way.": "একটি নেটওয়ার্ক ত্রুটির কারণে মিডিয়া ডাউনলোড আংশিকভাবে ব্যর্থ হয়েছে৷",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "মিডিয়া লোড করা যায়নি, হয় সার্ভার বা নেটওয়ার্ক ব্যর্থ হওয়ার কারণে বা ফর্ম্যাটটি সমর্থিত নয়।",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "মিডিয়া প্লেব্যাক একটি সমস্যার কারণে বা মিডিয়া ব্যবহার করা বৈশিষ্ট্যগুলি আপনার ব্রাউজার সমর্থন করে না বলে বাতিল করা হয়েছে৷",
  "No compatible source was found for this media.": "এই মিডিয়ার জন্য কোন সামঞ্জস্যপূর্ণ উৎস পাওয়া যায়নি.",
  "The media is encrypted and we do not have the keys to decrypt it.": "মিডিয়া এনক্রিপ্ট করা হয়েছে এবং এটি ডিক্রিপ্ট করার সমাধান আমাদের কাছে নেই।",
  "Play Video": "ভিডিও প্লে করুন",
  "Close": "বন্ধ করুন",
  "Close Modal Dialog": "মোডাল ডায়ালগ বন্ধ করুন",
  "Modal Window": "মোডাল উইন্ডো",
  "This is a modal window": "এটি একটি মোডাল উইন্ডো",
  "This modal can be closed by pressing the Escape key or activating the close button.": "Esc কী চেপে বা ক্লোজ বাটনটি সক্রিয় করে এই মডেলটি বন্ধ করা যেতে পারে।",
  ", opens captions settings dialog": ", ক্যাপশন সেটিংস ডায়ালগ খোলে",
  ", opens subtitles settings dialog": ", সাবটাইটেল সেটিংস ডায়ালগ খোলে",
  ", opens descriptions settings dialog": ", বর্ণনা সেটিংস ডায়ালগ খোলে",
  ", selected": ", নির্বাচিত",
  "captions settings": "ক্যাপশন সেটিংস",
  "subtitles settings": "সাবটাইটেল সেটিংস ",
  "descriptions settings": "বর্ণনা সেটিংস",
  "Text": "টেক্সট",
  "White": "সাদা",
  "Black": "কালো",
  "Red": "লাল",
  "Green": "সবুজ",
  "Blue": "নীল",
  "Yellow": "হলুদ",
  "Magenta": "ম্যাজেন্টা",
  "Cyan": "নীল সবুজ",
  "Background": "পটভূমি",
  "Window": "উইন্ডো",
  "Transparent": "স্বচ্ছ",
  "Semi-Transparent": "আধা-স্বচ্ছ",
  "Opaque": "অস্বচ্ছ",
  "Font Size": "অক্ষরের আকার",
  "Text Edge Style": "টেক্সট এজ স্টাইল",
  "None": "কোনোটিই নয়",
  "Raised": "বাড়ানো হয়েছে",
  "Depressed": "নামানো হয়েছে",
  "Uniform": "ইউনিফর্ম",
  "Drop shadow": "ড্রপশ্যাডো",
  "Font Family": "অক্ষরের পরিবার",
  "Proportional Sans-Serif": "সমানুপাতিক সানস-সেরিফ",
  "Monospace Sans-Serif": "মনোস্পেস সানস-সেরিফ",
  "Proportional Serif": "সমানুপাতিক সেরিফ",
  "Monospace Serif": "মনোস্পেস সেরিফ",
  "Casual": "ক্যাজুয়াল",
  "Script": "স্ক্রিপ্ট",
  "Small Caps": "ছোট ক্যাপস",
  "Reset": "রিসেট",
  "restore all settings to the default values": "সমস্ত সেটিংস ডিফল্ট মানগুলিতে পুনরুদ্ধার করুন",
  "Done": "সম্পন্ন",
  "Caption Settings Dialog": "ক্যাপশন সেটিংস ডায়ালগ",
  "Beginning of dialog window. Escape will cancel and close the window.": "ডায়লগ উইন্ডোর শুরু। Esc কী বাতিল করবে এবং উইন্ডো বন্ধ করবে।",
  "End of dialog window.": "ডায়ালগ উইন্ডোর শেষ।",
  "{1} is loading.": "{1} লোড হচ্ছে.",
  "Exit Picture-in-Picture": "পিকচার-ইন-পিকচার থেকে প্রস্থান করুন",
  "Picture-in-Picture": "পিকচার-ইন-পিকচার",
  "No content": "কোন বিষয়বস্তু নেই"
});