videojs.addLanguage('az', {
  "Audio Player": "Audiopleyer",
  "Video Player": "Videopleyer",
  "Play": "Oynat",
  "Pause": "Pauza",
  "Replay": "Yenidən oynat",
  "Current Time": "Cari Vaxt",
  "Duration": "Müddət",
  "Remaining Time": "Qalan vaxt",
  "Stream Type": "Yayım növü",
  "LIVE": "CANLI",
  "Seek to live, currently behind live": "Canlı yayım axtarışı, hal-hazırda canlı yayımdan geridədir",
  "Seek to live, currently playing live": "Canlı yayım axtarışı, hal-hazırda canlı yayım göstərilir",
  "Loaded": "Yükləndi",
  "Progress": "Yüklənmə",
  "Progress Bar": "Yüklənmə göstəricisi",
  "progress bar timing: currentTime={1} duration={2}": "{1} / {2}",
  "Fullscreen": "Tam ekran",
  "Exit Fullscreen": "Tam ekrandan çıx",
  "Mute": "Səssizi qoş",
  "Unmute": "Səssizi söndür",
  "Playback Rate": "Oynatma sürəti",
  "Subtitles": "Alt yazılar",
  "subtitles off": "Alt yazıları söndür",
  "Captions": "Başlıqlar",
  "captions off": "Başlıqları söndür",
  "Chapters": "Fəsillər",
  "Descriptions": "Təsvirlər",
  "descriptions off": "Təsvirləri söndür",
  "Audio Track": "Audio Trek",
  "Volume Level": "Səs Səviyyəsi",
  "You aborted the media playback": "Siz medianın oxudulmasını dayandırdınız",
  "A network error caused the media download to fail part-way.": "Şəbəkə xətası səbəbindən medianın endirilməsi yarıda qaldı.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "Yükləmə xətası.",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "Media faylının korlanması səbəbilə və ya media faylın brauzerinizin dəstəkləmədiyi funksiyalardan istifadə etdiyinə görə medianın oxudulması dayandırılıb.",
  "No compatible source was found for this media.": "Bu media üçün uyğun mənbə tapılmadı.",
  "The media is encrypted and we do not have the keys to decrypt it.": "Media faylı şifrələnib və onun şifrəsini açmaq üçün açarlar yoxdur.",
  "Play Video": "Videonu oynat",
  "Close": "Bağla",
  "Close Modal Dialog": "Modal dialoqunu bağla",
  "Modal Window": "Modal pəncərəsi",
  "This is a modal window": "Bu modal pəncərəsidir",
  "This modal can be closed by pressing the Escape key or activating the close button.": "Bu modal Escape düyməsini basmaqla və ya bağlama düyməsini sıxmaqla bağlana bilər.",
  ", opens captions settings dialog": ", başlıq parametrləri dialoqunu açır",
  ", opens subtitles settings dialog": ", altyazı parametrləri dialoqunu açır",
  ", opens descriptions settings dialog": ", təsvir parametrləri dialoqunu açır",
  ", selected": ", seçilmiş",
  "captions settings": "başlıq parametrləri",
  "subtitles settings": "altyazı parametrləri",
  "descriptions settings": "təsvir parametrləri",
  "Text": "Tekst",
  "White": "Ağ",
  "Black": "Qara",
  "Red": "Qırmızı",
  "Green": "Yaşıl",
  "Blue": "Göy",
  "Yellow": "Sarı",
  "Magenta": "Bənövşəyi",
  "Cyan": "Mavi",
  "Background": "Fon",
  "Window": "Pəncərə",
  "Transparent": "Şəffaf",
  "Semi-Transparent": "Yarım-Şəffaf",
  "Opaque": "Qeyri-şəffaf",
  "Font Size": "Şrift Ölçüsü",
  "Text Edge Style": "Mətnin kənarlarının üslubu",
  "None": "Heç biri",
  "Raised": "Artırılmış",
  "Depressed": "Azaldılmış",
  "Uniform": "Ümumiləşdirilmiş",
  "Drop shadow": "Arxa kölgə",
  "Font Family": "Şrift Ailəsi",
  "Proportional Sans-Serif": "Proporsional Sans-Serif",
  "Monospace Sans-Serif": "Birenli Sans-Serif",
  "Proportional Serif": "Proporsional Serif",
  "Monospace Serif": "Birenli Serif",
  "Casual": "Gündəlik",
  "Script": "Skript",
  "Small Caps": "Kiçik böyük hərflər",
  "Reset": "Sıfırla",
  "restore all settings to the default values": "bütün parametrləri susmaya görə bərpa edin",
  "Done": "Bitdi",
  "Caption Settings Dialog": "Başlıq Parametrləri Dialoqu",
  "Beginning of dialog window. Escape will cancel and close the window.": "Dialoq pəncərəsinin başlanğıcı. Escape düyməsi ləğv edəcək və pəncərəni bağlayacaq.",
  "End of dialog window.": "Dialoq pəncərəsinin sonu.",
  "{1} is loading.": "{1} yüklənir.",
  "Exit Picture-in-Picture": "Şəkil içində şəkil rejimindən çıxın",
  "Picture-in-Picture": "Şəkil içində şəkil rejimi",
  "No content": "Məzmun yoxdur",
  "Color": "Rəng",
  "Opacity": "Qeyri-şəffaflıq",
  "Text Background": "Tekst Fonu",
  "Caption Area Background": "Başlıq Sahəsinin Fonu",
  "Playing in Picture-in-Picture": "Şəkil içində şəkil rejimində oynayır",
  "Skip backward {1} seconds": "{1} saniyə geriyə keçin",
  "Skip forward {1} seconds": "{1} saniyə qabağa keçin"
});