<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Sabre\\Xml\\' => array($vendorDir . '/sabre/xml/lib'),
    'Sabre\\VObject\\' => array($vendorDir . '/sabre/vobject/lib'),
    'Sabre\\Uri\\' => array($vendorDir . '/sabre/uri/lib'),
    'Sabre\\HTTP\\' => array($vendorDir . '/sabre/http/lib'),
    'Sabre\\Event\\' => array($vendorDir . '/sabre/event/lib'),
    'Sabre\\' => array($vendorDir . '/sabre/dav/lib'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/Psr/Log'),
);
