:root{--panel-width:300px}#PDFBug,#PDFBug button,#PDFBug input,#PDFBug select{font:message-box}#PDFBug{--csstools-color-scheme--dark: ;background-color:#fff;border:1px solid #666;bottom:0;color:#000;color-scheme:only light;font-size:10px;padding:0;position:fixed;right:0;top:32px;width:300px;width:var(--panel-width)}#PDFBug .controls{background:#eee;border-bottom:1px solid #666;padding:3px}#PDFBug .panels{bottom:0;left:0;overflow:auto;position:absolute;right:0;top:27px}#PDFBug .panels>div{padding:5px}#PDFBug button.active{font-weight:700}.debuggerHideText:hover,.debuggerShowText{background-color:rgba(255,255,0,.25)}#PDFBug .stats{font-family:courier;font-size:10px;white-space:pre}#PDFBug .stats .title{font-weight:700}#PDFBug table{font-size:10px;white-space:pre}#PDFBug table.showText{border-collapse:collapse;text-align:center}#PDFBug table.showText,#PDFBug table.showText td,#PDFBug table.showText tr{border:1px solid #000;padding:1px}#PDFBug table.showText td.advance{color:grey}#viewer.textLayer-visible .textLayer{opacity:1}#viewer.textLayer-visible .canvasWrapper{background-color:#80ff80}#viewer.textLayer-visible .canvasWrapper canvas{mix-blend-mode:screen}#viewer.textLayer-visible .textLayer span{background-color:rgba(255,255,0,.1);border:1px solid rgba(255,0,0,.5);box-sizing:border-box;color:#000}#viewer.textLayer-visible .textLayer span[aria-owns]{background-color:rgba(255,0,0,.3)}#viewer.textLayer-hover .textLayer span:hover{background-color:#fff;color:#000}#viewer.textLayer-shadow .textLayer span{background-color:hsla(0,0%,100%,.6);color:#000}.pdfBugGroupsLayer{bottom:0;left:0;pointer-events:none;position:absolute;right:0;top:0}.pdfBugGroupsLayer>*{outline-color:red;outline-width:2px;position:absolute;--hover-outline-style:solid!important;--hover-background-color:rgba(255,0,0,.2)}.pdfBugGroupsLayer>:hover{background-color:var(--hover-background-color);cursor:pointer;outline-style:var(--hover-outline-style)}.showDebugBoxes .pdfBugGroupsLayer>*{outline-style:dashed}.showDebugBoxes .pdfBugGroupsLayer{pointer-events:all}