@page{size:auto;margin:1.4cm}@media print{thead{display:table-header-group}body,html{height:auto;min-height:0}#pagemargin{margin:5px;min-height:0}}blockquote,body,dd,div,dl,dt,fieldset,form,h1,h2,h3,h4,h5,h6,html,input,p,pre,td,th{margin:0;padding:0}body,html,img{border:0}body,html{height:auto;width:100%}address,caption,cite,code,dfn,th,var{font-style:normal;font-weight:400}caption,th{text-align:left}h1,h2,h3,h4,h5,h6{font-size:100%}q:after,q:before{content:""}.nowrap{white-space:nowrap}.wrap{white-space:normal}.fullwidth,table{width:100%}table{border-collapse:collapse;font-size:11px;text-align:left}table th{font-weight:700}table td,table th{padding:2px 3px 2px 0}.x-date-nextday a,.x-date-prevday a{color:transparent}.x-date-active a{color:#000;-webkit-text-decoration:none;text-decoration:none}.x-date-active{text-align:right}.x-date-left,.x-date-right{display:none}.x-btn button{background:none repeat scroll 0 0 transparent;border:0}th.date-header{background-color:#d3d3d3;text-align:right;width:50%}th.date-header-center{background-color:#d3d3d3;text-align:center;width:50%}table.print-calendar{border:1px solid #000;border-collapse:collapse;height:95%}.print-calendar th{border:1px solid #000}#top th,table.inner{border:none}.calendar-allday td{border:1px solid #000;text-align:center}.calendar-normal td{text-align:left}td.calendar-item{width:100%}.k-calendar-header{border:1px solid #000}table.k-calendar-days{margin-top:5px}table.k-calendar-days th{background-color:#d3d3d3;border:1px solid #000;text-align:center;width:14.28%!important}table.k-calendar-days td{border:1px solid #000;text-align:left;vertical-align:top}table.k-calendar-days td svg.k-appointment-status{bottom:0;display:inline-block;left:0;position:absolute;top:0;width:7px}table.k-calendar-days td .k-appointment{background-color:#fff;border-radius:4px;font-size:14px;height:20px;left:2.5%;line-height:1.5;margin-bottom:1px;position:absolute;width:90%}table.k-calendar-days td .k-overflow-indicator{bottom:0;display:inline-block;height:10px;position:absolute;right:0}